@@ -17,16 +17,6 @@ import pyvista as pv
 import mahotas as mh
 import numpy as np
 import vtk
-#from phenotastic.misc import autocrop
-#from phenotastic.mesh import remesh
-#from phenotastic.mesh import create_mesh, label_mesh
-#from phenotastic.misc import rand_cmap
-#from phenotastic.mesh import repair, make_manifold
-#from phenotastic.misc import flatten
-#import phenotastic.domains as boa
-#from phenotastic import mesh as mp
-# from phenotastic.mesh import get_connected_vertices_all
-# from img2org.quantify import get_layers
 
 def get_l1(seg_img, bg=0, selem=None):
     # TODO: Include option to remove cells bordering the image limits?
@@ -269,15 +259,17 @@ def merge_junctions_euclidean(j_indices, j_neighs, r):
     matches = tree.query_ball_tree(tree, r=r)
     to_merge = merge(matches)
     to_remove = []
+    # j_neighs = np.array([vv for vv in j_neighs])
+    j_neighs = list(j_neighs)
     for group in to_merge:
         group = np.asarray(list(group))
-        j_neighs[group[0]] = np.unique(np.hstack(j_neighs[group]))
+        j_neighs[group[0]] = np.unique(np.hstack([j_neighs[gg] for gg in group]))
         j_indices[group[0]] = np.mean(j_indices[group], 0)
         to_remove.extend(list(group[1:]))
         
     to_keep = np.logical_not(np.isin(np.arange(j_indices.shape[0]), to_remove))
     j_indices = j_indices[to_keep]
-    j_neighs = j_neighs[to_keep]
+    j_neighs = np.array(j_neighs)[to_keep]
     return j_indices, j_neighs    
 
 
@@ -291,6 +283,8 @@ def find_junctions(image, background=0, threshold=3, merge_adjacent=True, resolu
 
     if include_set == 'l1':
         include = get_layers(image, bg=background, depth=1)[0]
+    else:
+        include= include_set
 
     # get the distances to the neighbours
     neighbours = connectivity.copy()
@@ -598,12 +592,12 @@ def proj_to_plane(point, v1, v2, v3):
     return proj
 
 
-def internal_angles(j_indices, j_neighs, edges, cells=None, background=0):
+def internal_angles(j_indices, j_neighs, edges, cells=None, background=0, mode='project'):
     poly = create_polydata(j_indices, edges)
     neighs = get_connected_vertices_all(poly, include_self=False)
     
     if cells is None:
-        # write convenience function for this
+        # TODO: write convenience function for this
         _, _, cells = center_triangulation(j_indices, j_neighs, background)
         cells = np.unique(cells)
 
@@ -617,20 +611,25 @@ def internal_angles(j_indices, j_neighs, edges, cells=None, background=0):
             if n_neighs_in_cell != 2:
                 print('Warning: vertex {ii} has too many neighbours in the same cell. Ignoring...')
                 continue
-
-            if n_neighs == 2:
-                # define plane using only neighbouring same-cell vertices
-                phi = angle(j_indices[v_neighs_in_cell[0]], 
-                              j_indices[jj], 
-                              j_indices[v_neighs_in_cell[1]])
-            else:
-                # define plane using all neighbouring vertices
-                phi = angle(j_indices[v_neighs_in_cell[0]], 
-                              proj_to_plane(j_indices[jj], 
-                                            j_indices[v_neighs_in_cell[0]], 
-                                            j_indices[v_neighs_in_cell[1]], 
-                                            np.mean(j_indices[v_neighs], 0)), 
-                              j_indices[v_neighs_in_cell[1]])
+            v1 = j_indices[v_neighs_in_cell[0]] - j_indices[jj]
+            v1 = v1 / np.linalg.norm(v1)
+            v2 = j_indices[v_neighs_in_cell[1]] - j_indices[jj]
+            v2 = v2 / np.linalg.norm(v2)
+            phi = np.arccos(np.dot(v1, v2)) / (2 * np.pi) * 360
+            # if n_neighs == 2:
+            #     # define plane using only neighbouring same-cell vertices
+            #     phi = angle(j_indices[v_neighs_in_cell[0]], 
+            #                   j_indices[jj], 
+            #                   j_indices[v_neighs_in_cell[1]])
+            # else:
+            #     # define plane using all neighbouring vertices
+                
+            #     phi = angle(j_indices[v_neighs_in_cell[0]], 
+            #                   proj_to_plane(j_indices[jj], 
+            #                                 j_indices[v_neighs_in_cell[0]], 
+            #                                 j_indices[v_neighs_in_cell[1]], 
+            #                                 np.mean(j_indices[v_neighs], 0)), 
+            #                   j_indices[v_neighs_in_cell[1]])
             output.append((cell, jj, v_neighs_in_cell[0], v_neighs_in_cell[1], phi, n_neighs))
     output = np.asarray(output)
     return output
